# Generated by roxygen2: do not edit by hand

export(cvmideal)
export(dmideal)
export(dvmgeom)
export(dvmideal)
export(freq_quantile)
export(load_vmdb_magnitudes)
export(load_vmdb_rates)
export(pmideal)
export(pvmgeom)
export(pvmideal)
export(qmideal)
export(qvmgeom)
export(qvmideal)
export(rmideal)
export(rvmgeom)
export(rvmideal)
export(select_knots)
export(vmgeom_vst_from_magn)
export(vmgeom_vst_to_r)
export(vmideal_vst_from_magn)
export(vmideal_vst_to_psi)
export(vmperception)
export(vmtable)
import(httr2)
