#' @name PER_2015_rates
#' @docType data
#' @keywords data
#' @title Visual rate observations of Perseids from 2015
#' @description
#' Visual rate observations of the Perseid shower from 2015.
#' @format A list with the same structure as returned by [vismeteor::load_vmdb_rates].
#' @details
#' `PER_2015_rates` are rate observations loaded with [vismeteor::load_vmdb_rates].
#' @seealso [vismeteor::load_vmdb]
NULL

#' @name PER_2015_magn
#' @docType data
#' @keywords data
#' @title Visual magnitude observations of Perseids from 2015
#' @description
#' Visual magnitude observations of the Perseid shower from 2015.
#' @format A list with the same structure as returned by [vismeteor::load_vmdb_magnitudes].
#' @details
#' `PER_2015_magn` are magnitude observations loaded with
#' [vismeteor::load_vmdb_magnitudes].
#' @seealso [vismeteor::load_vmdb]
NULL
